package com.supervision.chat.controller;

import cn.hutool.json.JSONUtil;
import com.supervision.chat.UploadParamEnum;
import com.supervision.chat.client.CustomMultipartFile;
import com.supervision.chat.client.LangChainChatService;
import com.supervision.chat.client.dto.CreateBaseDTO;
import com.supervision.chat.client.dto.DeleteFileDTO;
import com.supervision.chat.client.dto.LangChainChatRes;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.service.ModelCaseService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

@Slf4j
@RequestMapping("chat/test/")
@RestController
@RequiredArgsConstructor
public class TestController {

    private final LangChainChatService langChainChatClient;

    private final ModelCaseService modelCaseService;

    @GetMapping("test")
    public void test() {
        CreateBaseDTO createBaseDTO = new CreateBaseDTO();
        createBaseDTO.setKnowledge_base_name("11111111");
        LangChainChatRes chat = langChainChatClient.createBase(createBaseDTO);
        log.info(JSONUtil.toJsonStr(chat));
    }

    @GetMapping("createExistCase")
    public void createExistCase(){
        List<ModelCase> list = modelCaseService.list();
        for (ModelCase modelCase : list) {
            LangChainChatRes chat = langChainChatClient.createBase(CreateBaseDTO.create(modelCase.getCaseNo()));
            log.info(JSONUtil.toJsonStr(chat));
        }
    }

    @PostMapping("uploadFile")
    public void testUploadFile(@RequestPart("file") MultipartFile file) throws IOException {
        CustomMultipartFile mockMultipartFile = new CustomMultipartFile(file.getOriginalFilename(), file.getInputStream());
        LangChainChatRes langChainChatRes = langChainChatClient.uploadFile("11111111",
                mockMultipartFile,
                "问讯笔录",
                UploadParamEnum.to_vector_store.getBooleanValue(),
                UploadParamEnum.override.getBooleanValue(),
                UploadParamEnum.not_refresh_vs_cache.getBooleanValue(),
                UploadParamEnum.chunk_size.getIntValue(),
                UploadParamEnum.chunk_overlap.getIntValue(),
                UploadParamEnum.zh_title_enhance.getBooleanValue(),
                UploadParamEnum.docs.getStrValue());
        log.info(JSONUtil.toJsonStr(langChainChatRes));
    }

    @GetMapping("deleteFile")
    public void testDeleteFile(String knowledgeBaseName, String fileName) {
        LangChainChatRes langChainChatRes = langChainChatClient.deleteFile(DeleteFileDTO.create(knowledgeBaseName, fileName));
        log.info(JSONUtil.toJsonStr(langChainChatRes));
    }

    @GetMapping("deleteBase")
    public void deleteBase(String knowledgeBaseName) {
        LangChainChatRes langChainChatRes = langChainChatClient.deleteBase(knowledgeBaseName);
        log.info(JSONUtil.toJsonStr(langChainChatRes));
    }

}
