package com.supervision.neo4j.utils;

import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.dto.ResultDTO;
import net.sf.jsqlparser.expression.LongValue;
import org.neo4j.driver.Record;
import org.neo4j.driver.Result;
import org.neo4j.driver.Value;
import org.neo4j.driver.internal.value.IntegerValue;
import org.neo4j.driver.internal.value.NullValue;
import org.neo4j.driver.internal.value.StringValue;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class Neo4jUtils {

    public static final String NODE_RETURN = " RETURN id(n) as id, n.name as name, n.nodeType as nodeType, n.recordId as recordId, n.caseId as caseId, n.picType as picType";
    public static final String REL_RETURN = " RETURN id(rel) as id, a.name as source, id(a) as sourceId, type(rel) as name, b.name as target, id(b) as targetId";

    public static CaseNode getOneNode(Result run) {
        CaseNode node = null;
        if (run.hasNext()) {
            Record record = run.next();
            long id = record.get("id").asLong();
            String name = record.get("name").asString();
            String nodeType = record.get("nodeType").asString();
            String recordId = record.get("recordId").asString();
            String caseId = record.get("caseId").asString();
            String picType = record.get("picType").asString();
            node = new CaseNode(id, name, nodeType, recordId, caseId, picType);
        }
        return node;
    }

    public static List<CaseNode> getNodeList(Result run) {
        List<CaseNode> list = new ArrayList<>();
        while (run.hasNext()) {
            Record record = run.next();
            long id = record.get("id").asLong();
            String name = record.get("name").asString();
            String nodeType = record.get("nodeType").asString();
            String recordId = record.get("recordId").asString();
            String caseId = record.get("caseId").asString();
            String picType = record.get("picType").asString();
            list.add(new CaseNode(id, name, nodeType, recordId, caseId, picType));
        }
        return list;
    }

    public static Rel getOneRel(Result run) {
        Rel rel = null;
        if (run.hasNext()) {
            Record record = run.next();
            long id = record.get("id").asLong();
            String source = record.get("source").asString();
            Long sourceId = record.get("sourceId").asLong();
            String name = record.get("name").asString();
            String target = record.get("target").asString();
            Long targetId = record.get("targetId").asLong();
            rel = new Rel(id, source, sourceId, name, target, targetId);
        }
        return rel;
    }

    public static List<Rel> getRelList(Result run) {
        List<Rel> list = new ArrayList<>();
        while (run.hasNext()) {
            Record record = run.next();
            long id = record.get("id").asLong();
            String source = record.get("source").asString();
            Long sourceId = record.get("sourceId").asLong();
            String name = record.get("name").asString();
            String target = record.get("target").asString();
            Long targetId = record.get("targetId").asLong();
            list.add(new Rel(id, source, sourceId, name, target, targetId));
        }
        return list;
    }

    public static List<ResultDTO> getResultDTOList(Result result) {


        List<ResultDTO> list = new ArrayList<>();
        while (result.hasNext()) {
            Record record = result.next();
            ResultDTO resultDTO = new ResultDTO();
            resultDTO.setStartId(valueTransportString(record.get("startId")));
            resultDTO.setEndId(valueTransportString(record.get("endId")));
            resultDTO.setRelId(valueTransportString(record.get("relId")));
            resultDTO.setRelId(valueTransportString(record.get("relId")));
            // 这里注意,neo4j的recordId对应的是note_record_split的数据
            resultDTO.setRecordSplitId(valueTransportString(record.get("recordSplitId")));
            // recordsId对应数据库note_record
            resultDTO.setRecordId(valueTransportString(record.get("recordId")));
            list.add(resultDTO);
        }
        return list;
    }

    /**
     * neo4j类型转换
     */
    public static String valueTransportString(Value value) {
        if (value instanceof NullValue) {
            return null;
        } else if (value instanceof IntegerValue) {
            return String.valueOf(value.asInt());
        } else if (value instanceof LongValue) {
            return String.valueOf(value.asLong());
        } else if (value instanceof StringValue) {
            return value.asString();
        }
        throw new RuntimeException("TODO 需要的类型自己写");

    }

}
