package com.supervision.police.domain;

import java.time.LocalDateTime;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import java.io.Serializable;

/**
 * 案件表(ModelCase)表实体类
 *
 * @author qmy
 * @since 2024-07-02 14:37:15
 */
@Data
@TableName("model_case")
public class ModelCase implements Serializable {
    
    /**
     * 主键
     */
    @TableId
    private String id;
    
    /**
     * 案件名称
     */
    private String caseName;
    
    /**
     * 案件类型
     */
    private String caseType;
    @TableField(exist=false)
    private String caseTypeName;

    /**
     * 案件状态
     */
    private String caseStatus;
    @TableField(exist=false)
    private String caseStatusName;

    /**
     * 案件数据状态  0:没有基础数据 1:数据未更新 2：数据已更新
     */
    private int caseDataStatus;

    /**
     * 案件分析状态  0:未分析 1：正在分析 2：分析成功  3：分析失败
     */
    private int caseAnalysisStatus;
    
    /**
     * 作案方式
     */
    private String crimeMode;
    @TableField(exist=false)
    private String crimeModeName;
    
    /**
     * 案件概述
     */
    private String caseDetail;
    
    /**
     * 立案时间
     */
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date registerTime;
    
    /**
     * 受理时间
     */
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date acceptTime;
    
    /**
     * 行为人
     */
    private String lawActor;
    
    /**
     * 当事人
     */
    private String lawParty;
    
    /**
     * 认定结果
     */
    private String identifyResult;
    @TableField(exist=false)
    private String identifyResultName;
    
    /**
     * 综合得分
     */
    private Integer totalScore;
    
    /**
     * 排序序号
     */
    private Integer indexNum;

    @TableField(exist=false)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateStartTime;


    @TableField(exist=false)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateEndTime;

    /**
     * 分析成功时间
     */
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime caseAnalysisSuccessTime;
    
    /**
     * 数据状态
     */
    private String dataStatus;

    /**
     * 案件编号
     */
    private String caseNo;


    /**
     * 创建人ID
     */
    private String createUserId;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime createTime;

    /**
     * 更新人
     */
    private String updateUserId;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateTime;

}

