package com.supervision.police.dto;

import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.supervision.minio.domain.MinioFile;
import com.supervision.police.domain.NoteRecord;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.*;

@Data
public class NoteRecordDetailDTO {

    private String id;

    /**
     * 案件id
     */
    private String caseId;

    /**
     * 笔录名称
     */
    private String recordName;

    /**
     * 笔录文件信息
     */
    private List<NoteRecordFileDTO> fileList = new ArrayList<>();

    /**
     * 笔录人员id
     */
    private String personId;

    /**
     * 身份证号码
     */
    private String personIdCard;

    /**
     * 姓名
     */
    private String name;

    /**
     * 角色
     */
    private String role;

    /**
     * 角色名称
     */
    private String roleName;

    /**
     * 供述材料
     */
    private Integer confessionMaterial;

    /**
     * 询问人
     */
    private String lawAsker;

    /**
     * 供述开始时间
     */
    private String confessionStartTime;

    /**
     * 供述结束时间
     */
    private String confessionEndTime;

    /**
     * 进度百分比
     */
    private String percentage;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    private List<NoteRecordDetailDTO> children = new ArrayList<>();

    public NoteRecordDetailDTO() {
    }

    public NoteRecordDetailDTO(NoteRecordDTO noteRecordDTO, Map<String, MinioFile> fileMap) {
        if (Objects.isNull(noteRecordDTO)){
            return;
        }
        this.id = noteRecordDTO.getId();
        this.caseId = noteRecordDTO.getCaseId();
        this.recordName = noteRecordDTO.getRecordName();
        this.personId = noteRecordDTO.getCasePersonId();
        this.name = noteRecordDTO.getName();
        this.role = noteRecordDTO.getRole();
        this.confessionMaterial = noteRecordDTO.getConfessionMaterial();
        this.lawAsker = noteRecordDTO.getLawAsker();
        this.confessionStartTime = noteRecordDTO.getConfessionStartTime();
        this.confessionEndTime = noteRecordDTO.getConfessionEndTime();
        this.createTime = noteRecordDTO.getCreateTime();
        this.personIdCard = noteRecordDTO.getCasePersonIdCard();
        if (StrUtil.isNotEmpty(noteRecordDTO.getFileIds())){
            this.fileList = Arrays.stream(noteRecordDTO.getFileIds().split(",")).map(fileId->{
                MinioFile minioFile = fileMap.get(fileId);
                if (Objects.isNull(minioFile)){
                    return null;
                }
                return new NoteRecordFileDTO(this.id,minioFile);
            }).filter(Objects::nonNull).toList();
        }else {
            this.fileList = new ArrayList<>(1);
        }
    }
}
