package com.supervision.police.dto.caseScore;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.List;

@Data
@Slf4j
public class CaseScoreDetailDTO {

    @Schema(description = "案件名称")
    private String caseName;

    @Schema(description = "评估时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime evaluationTime;

    @Schema(description = "共性指标分数")
    private Integer commonScore;

    @Schema(description = "特定指标名 入罪指标/出罪指标")
    private String specificCrime;

    @Schema(description = "入罪/出罪指标分数")
    private Integer specificCrimeScore;

    @Schema(description = "总分数")
    private Integer totalScore;

    @Schema(description = "分数描述")
    private String scoreDesc;

    @Schema(description = "司法定性")
    private String judicialCharacterization;

    @Schema(description = "共性指标数量")
    private Integer commonIndexCount;

    @Schema(description = "共性指标命中数量")
    private Integer commonIndexHitCount;

    @Schema(description = "共性指标命描述(指标+命中的原子指标)")
    private List<String> commonIndexDescList;

    @Schema(description = "入罪/出罪 0:入罪 1：出罪")
    private Integer crimeOrOut;

    @Schema(description = "入罪/出罪指标数量")
    private Integer specificCrimeIndexCount;

    @Schema(description = "入罪/出罪指标命中数量")
    private Integer specificCrimeIndexHitCount;

    @Schema(description = "入罪/出罪指标命描述(指标+命中的原子指标)")
    private List<String> specificCrimeIndexDescList;

    @Schema(description = "缺失的原子指标数量")
    private Long missAtomicIndexCount;

    // 公共指标描述
    private String commonIndexDesc;

    // 出罪\入罪指标描述
    private String specificCrimeIndexDesc;


    public void buildDesc(){
        if (CollUtil.isNotEmpty(this.getCommonIndexDescList())){
            StringBuilder append = new StringBuilder();
            for (int i = 0; i < this.getCommonIndexDescList().size(); i++) {
                String text = this.getCommonIndexDescList().get(i);
                if (!StrUtil.endWith(text,"。")){
                    text = text + "。";
                }
                if (i > 0){
                    append.append("\t");
                }
                append.append(i+1).append(".\t").append(text);
                if (i < this.getCommonIndexDescList().size()-1){
                    append.append("\n");
                }
            }
            this.setCommonIndexDesc(append.toString());

            append = new StringBuilder();
            for (int i = 0; i < this.getSpecificCrimeIndexDescList().size(); i++) {
                String text = this.getSpecificCrimeIndexDescList().get(i);
                if (!StrUtil.endWith(text,"。")){
                    text = text + "。";
                }
                if (i > 0){
                    append.append("\t");
                }
                append.append(i+1).append(".\t").append(text);
                if (i < this.getSpecificCrimeIndexDescList().size()-1){
                    append.append("\n");
                }
            }
            this.setSpecificCrimeIndexDesc(append.toString());
        }
    }
}
