package com.supervision.police.dto.caseScore;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.police.domain.ModelAtomicIndex;
import com.supervision.police.domain.ModelAtomicResult;
import com.supervision.police.domain.ModelIndex;
import com.supervision.police.domain.ModelIndexResult;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatClient;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description 指标结果
 */
@Data
@Slf4j
public class IndexResult {

    @Schema(description = "指标id")
    private String indexId;

    @Schema(description = "指标名称")
    private String indexName;

    @Schema(description = "指标分组 1:共性指标 2：入罪指标 3：出罪指标")
    private String indexGroup;

    @Schema(description = "指标分数")
    private Integer indexScore;

    @Schema(description = "实际分数")
    private int actualScore;

    @Schema(description = "原子指标结果")
    private List<AtomicResult> atomicResults = new ArrayList<>();

    public IndexResult() {
    }

    public IndexResult(ModelIndex indexInfo, ModelIndexResult modelIndexResult,
                       List<ModelAtomicIndex> atomicIndexList, List<ModelAtomicResult> atomicResultList ) {
        if (Objects.isNull(modelIndexResult) || Objects.isNull(indexInfo)){
            return;
        }
        this.indexId = indexInfo.getId();
        this.indexGroup = indexInfo.getIndexType();
        this.indexName = indexInfo.getName();
        this.indexScore = indexInfo.getIndexScore();
        if (StrUtil.equals("true", modelIndexResult.getIndexResult())){
            // 如果运行结果为真，则实际分数为指标分数
            this.actualScore = indexInfo.getIndexScore();
        }
        if (CollUtil.isEmpty(atomicResultList) || StrUtil.isEmpty(modelIndexResult.getAtomicIds())){
            return;
        }

        // 填充原子指标结果属性
        List<String> atomicIdList = Arrays.stream(modelIndexResult.getAtomicIds().split(",")).toList();
        atomicResultList = atomicResultList.stream().filter(atomicResult -> atomicIdList.contains(atomicResult.getAtomicId())).distinct().toList();
        if (CollUtil.isEmpty(atomicResultList)){
            log.info("IndexResult:经过过滤后的原子指标结果为空");
            return;
        }
        if (Objects.isNull(atomicIndexList)){
            atomicIndexList = new ArrayList<>(1);
        }
        // 注意不要有重复值
        Map<String, ModelAtomicIndex> atomicIndexMap = atomicIndexList.stream().collect(Collectors.toMap(ModelAtomicIndex::getId, k -> k, (k1, k2) -> k1));
        this.atomicResults = atomicResultList.stream().map(atomicResult -> new AtomicResult(indexId,
                atomicIndexMap.get(atomicResult.getAtomicId()), atomicResult)).toList();

    }

    /**
     * 构建指标结果描述
     * @return
     */
    public String buildDesc(ChatClient chatClient){
        return indexName;
    }

}
