package com.supervision.police.dto.user;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.police.domain.SystemUser;
import com.supervision.police.dto.RoleDTO;
import com.supervision.utils.TokenUtil;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


@Data
public class LoginResVO {

    @Schema(description = "主键")
    private String id;

    @Schema(description = "用户账户")
    private String account;

    @Schema(description = "用户名称")
    private String userName;

    @Schema(description = "用户角色列表")
    private List<UserRoleDTO> roleList;

    @Schema(description = "菜单权限标识")
    private List<String> permission = new ArrayList<>();

    @Schema(description = "token")
    private String token;

    public LoginResVO() {
    }

    public LoginResVO(SystemUser systemUser) {
        if (Objects.isNull(systemUser)){
            return;
        }
        this.id = systemUser.getId();
        this.account = systemUser.getAccount();
        this.userName = systemUser.getUserName();
    }

    public void setToken(UserInfoDTO userInfoDTO) {
        if (Objects.isNull(userInfoDTO)){
            return;
        }
        if (CollUtil.isNotEmpty(this.roleList)){
            userInfoDTO.setUserRoleList(this.roleList);
        }
        if (CollUtil.isNotEmpty(this.permission)){
            userInfoDTO.setPermission(this.permission);
        }
        this.token = TokenUtil.creatToken(JSONUtil.toJsonStr(userInfoDTO));
    }
    public void setToken(String token) {
        this.token = token;
    }

    public void setPermission(List<String> permission) {
        this.permission = permission;
    }

    public void roleMenuDTOSetPermission(List<RoleMenuDTO> roleMenuDTOS) {
        if (CollUtil.isEmpty(roleMenuDTOS)){
            return;
        }
        this.permission = roleMenuDTOS.stream().map(RoleMenuDTO::getLabelCode).filter(Objects::nonNull)
                .distinct().collect(Collectors.toList());
    }
}
