package com.supervision.police.dto.user;

import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.supervision.police.domain.SystemUser;
import com.supervision.police.domain.SystemUserRoleRelation;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.*;

@Data
public class UserInfoDTO {


    @Schema(description = "用户id")
    private String id;

    @Schema(description = "头像id")
    private String headPicId;

    @Schema(description = "账户名")
    private String account;

    @Schema(description = "用户名")
    private String userName;

    @Schema(description = "手机号码")
    private String phoneNum;

    @Schema(description = "账号状态 0正常 1停用")
    private Integer status;

    @Schema(description = "备注")
    private String remark;


    @Schema(description = "用户角色列表")
    private List<UserRoleDTO> userRoleList = new ArrayList<>();

    @Schema(description = "菜单权限标识")
    private List<String> permission = new ArrayList<>();

    @Schema(description = "最近登录时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private LocalDateTime recentLoginTime;

    @Schema(description = "注册时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private LocalDateTime registerTime;

    public UserInfoDTO() {
    }

    public UserInfoDTO(SystemUser systemUser) {
        systemUserSetter(systemUser);
    }

    public UserInfoDTO(SystemUser systemUser, Map<String, List<UserRoleDTO>> userRoleMap) {
        systemUserSetter(systemUser);
        if (StrUtil.isNotEmpty(this.id)){
            this.setUserRoleList(userRoleMap.getOrDefault(this.id, new ArrayList<>(1)));
        }
    }

    private void systemUserSetter(SystemUser systemUser){
        if (Objects.isNull(systemUser)){
            return;
        }
        this.id = systemUser.getId();
        this.headPicId = systemUser.getHeadPicId();
        this.account = systemUser.getAccount();
        this.userName = systemUser.getUserName();
        this.phoneNum = systemUser.getPhoneNum();
        this.status = systemUser.getStatus();
        this.recentLoginTime = systemUser.getRecentLoginTime();
        this.registerTime = systemUser.getCreateTime();
        this.remark = systemUser.getRemark();
    }

}
