package com.supervision.police.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.dto.AtomicIndexDTO;
import com.supervision.police.dto.IndexDetail;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 案件表(ModelCase)表数据库访问层
 *
 * @author qmy
 * @since 2024-07-02 14:47:59
 */
public interface ModelCaseMapper extends BaseMapper<ModelCase> {

    IPage<ModelCase> selectAll(IPage<ModelCase> iPage, ModelCase modelCase);

    int selectMaxIndex();

    /**
     * 查询案件指标详情
     * @param iPage 分页参数
     * @param caseId 指标id
     * @param indexType 指标类型
     * @return 通过指标数据和指标结果进行关联，然后进行过滤查询
     */
    IPage<IndexDetail> getCaseIndexDetail(IPage<IndexDetail> iPage,
                                          @Param("caseId") String caseId,
                                          @Param("indexType") String indexType);

    /**
     * 查询指标详情
     * @param iPage 分页参数
     * @param caseId 案件id 必填
     * @param indexType 指标类型 必填
     * @return 以指标数据为基础 拼接指标结果数据
     */
    IPage<IndexDetail> getIndexDetail(IPage<IndexDetail> iPage,
                                          @Param("caseId") String caseId,
                                          @Param("indexType") String indexType);

    List<AtomicIndexDTO> getAtomicDetail(@Param("caseId") String caseId,
                                         @Param("atomicIds") List<String> atomicIds);

}

