package com.supervision.police.service.impl;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.supervision.constant.CaseAnalysisStatusEnum;
import com.supervision.constant.CaseDataStatusEnum;
import com.supervision.constant.ScoreEnum;
import com.supervision.police.domain.CasePerson;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.mapper.CasePersonMapper;
import com.supervision.police.mapper.ModelCaseMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.service.CaseStatusManageService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class CaseStatusManageServiceImpl implements CaseStatusManageService {


    private final ModelCaseMapper modelCaseMapper;

    private final NoteRecordMapper noteRecordMapper;

    private final CasePersonMapper casePersonMapper;

    @Override
    public void whenSaveCasePeople(String caseId, CasePerson casePerson) {
        Assert.notEmpty(caseId, "案件id不能为空");
        Assert.notEmpty(casePerson.getRoleCode(), "角色id不能为空");

        ModelCase modelCase = modelCaseMapper.selectById(caseId);
        Assert.notNull(modelCase,"案件信息不存在");

        if ("1".equals(casePerson.getRoleCode())){
            // 保存行为人数据
            List<CasePerson> casePersonList = casePersonMapper.selectList(
                    new LambdaQueryWrapper<CasePerson>().eq(CasePerson::getCaseId, caseId).eq(CasePerson::getRoleCode, "1"));
            String lawActor = casePersonList.stream().map(CasePerson::getName).collect(Collectors.joining(","));

            modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                    .eq(ModelCase::getId, caseId).set(ModelCase::getLawActor, lawActor));
        }else {
            // 保存当事人数据
            List<CasePerson> casePersonList = casePersonMapper.selectList(
                    new LambdaQueryWrapper<CasePerson>().eq(CasePerson::getCaseId, caseId).ne(CasePerson::getRoleCode, "1"));
            String lawParty = casePersonList.stream().map(CasePerson::getName).collect(Collectors.joining(","));
            modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                    .eq(ModelCase::getId, caseId).set(ModelCase::getLawParty, lawParty));
        }
    }

    @Override
    public void whenDeleteCaseIndex(String caseId) {

    }

    @Override
    public void whenUploadRecord(String caseId) {
        // 存在行为人的笔录信息
        Long count = noteRecordMapper.selectCount(
                new LambdaQueryWrapper<>(new NoteRecord())
                        .eq(NoteRecord::getCaseId, caseId).eq(NoteRecord::getRole,"行为人"));

        if (count > 0){
            modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                    .eq(ModelCase::getId, caseId)
                    .set(ModelCase::getCaseDataStatus, CaseDataStatusEnum.UPDATE.getCode())
            );
        }

    }

    @Override
    public void whenDeleteRecord(String caseId) {
        this.whenUpdateData(caseId);
    }

    @Override
    public void whenUpdateData(String caseId) {
        Assert.notEmpty(caseId, "案件id不能为空");
        modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                .eq(ModelCase::getId, caseId)
                .set(ModelCase::getCaseAnalysisStatus, CaseAnalysisStatusEnum.UNANALYZED.getCode())
                .set(ModelCase::getCaseDataStatus, CaseDataStatusEnum.UPDATE.getCode())
        );
    }

    @Override
    public void whenAnalyseCaseSuccess(String caseId, Integer score) {

        Assert.notEmpty(caseId, "案件id不能为空");

        modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                .eq(ModelCase::getId, caseId)
                .set(ModelCase::getCaseAnalysisStatus, CaseAnalysisStatusEnum.ANALYZED.getCode())
                .set(ModelCase::getCaseDataStatus, CaseDataStatusEnum.NOT_UPDATE.getCode())
                .set(ModelCase::getTotalScore, score)
                .set(ModelCase::getIdentifyResult, ScoreEnum.getScoreEnum(score).getCode())
                .set(ModelCase::getCaseAnalysisSuccessTime, LocalDateTime.now()));
    }

    @Override
    public void whenAnalyseCaseFail(String caseId) {

        Assert.notEmpty(caseId, "案件id不能为空");

        modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                .eq(ModelCase::getId, caseId)
                .set(ModelCase::getCaseAnalysisStatus, CaseAnalysisStatusEnum.ANALYZE_FAIL.getCode()));

    }

    @Override
    public void whenAnalyseCaseStart(String caseId) {

        Assert.notEmpty(caseId, "案件id不能为空");

        modelCaseMapper.update(new LambdaUpdateWrapper<ModelCase>()
                .eq(ModelCase::getId, caseId)
                .set(ModelCase::getCaseAnalysisStatus, CaseAnalysisStatusEnum.ANALYZING.getCode()));

    }

    @Override
    public void whenUpdateEvidence(String caseId) {

        whenUpdateData(caseId);
    }

    @Override
    public void whenUpdateIndexResult(String caseId) {
        whenUpdateData(caseId);
    }
}
