package com.supervision.police.service.impl;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.chat.UploadParamEnum;
import com.supervision.chat.client.CustomMultipartFile;
import com.supervision.chat.client.LangChainChatService;
import com.supervision.chat.client.dto.DeleteFileDTO;
import com.supervision.chat.client.dto.LangChainChatRes;
import com.supervision.common.utils.StringUtils;
import com.supervision.config.BusinessException;
import com.supervision.minio.domain.MinioFile;
import com.supervision.minio.service.MinioService;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.dto.NoteRecordDTO;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.service.ModelCaseService;
import com.supervision.police.service.NoteRecordService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Slf4j
@Service
@RequiredArgsConstructor
public class NoteRecordServiceImpl extends ServiceImpl<NoteRecordMapper, NoteRecord>
        implements NoteRecordService {

    private final LangChainChatService langChainChatService;

    private final ModelCaseService modelCaseService;

    private final MinioService minioService;

    @Override
    public List<NoteRecordDTO> selectNoteRecordDTOList(NoteRecord noteRecord) {
        return super.getBaseMapper().selectNoteRecordDTOList(noteRecord);
    }

    public void uploadFileToLangChainChat(String caseId) {
        List<NoteRecord> recordList = this.lambdaQuery().eq(NoteRecord::getCaseId, caseId).list();
        ModelCase modelCase = modelCaseService.getById(caseId);
        if (StringUtils.isEmpty(modelCase.getCaseNo())) {
            log.info("案件编号不能为空");
            return;
        }
        // 查询知识库中已经存在的文件
        LangChainChatRes<List<String>> queryFileListRes = langChainChatService.queryFileList(modelCase.getCaseNo());
        if (!queryFileListRes.getCode().equals(200)) {
            log.info("获取知识库已存在的文件失败");
        }
        HashSet<String> knowledgeFileNameSet = new HashSet<>(queryFileListRes.getData());
        Set<String> databaseFileNameSet = new HashSet<>();
        for (NoteRecord record : recordList) {
            for (String fileId : record.getFileIds().split(",")) {
                MinioFile minioFile = minioService.getMinioFile(fileId);
                InputStream inputStream = null;
                databaseFileNameSet.add(minioFile.getFilename());
                if (!knowledgeFileNameSet.contains(minioFile.getFilename())){
                    try {
                        inputStream = minioService.getObjectInputStream(minioFile);
                        // 这里需要把文件传输,传输到知识库中去
                        CustomMultipartFile mockMultipartFile = new CustomMultipartFile(minioFile.getFilename(), inputStream);
                        LangChainChatRes langChainChatRes = langChainChatService.uploadFile(modelCase.getCaseNo(),
                                mockMultipartFile,
                                "问讯笔录",
                                UploadParamEnum.to_vector_store.getBooleanValue(),
                                UploadParamEnum.override.getBooleanValue(),
                                UploadParamEnum.not_refresh_vs_cache.getBooleanValue(),
                                UploadParamEnum.chunk_size.getIntValue(),
                                UploadParamEnum.chunk_overlap.getIntValue(),
                                UploadParamEnum.zh_title_enhance.getBooleanValue(),
                                UploadParamEnum.docs.getStrValue());
                        if (!langChainChatRes.getCode().equals(200)) {
                            log.error("上传文件到知识库失败:{}", langChainChatRes.getMsg());
                        }else {
                            knowledgeFileNameSet.add(minioFile.getFilename());
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        log.error("从minio中获取文件失败:{}", e.getMessage());
                        continue;
                    }
                }

            }
        }
        // 遍历完成,删除知识库有,数据库没有的文件
        for (String fileName : knowledgeFileNameSet) {
            if (!databaseFileNameSet.contains(fileName)) {
                LangChainChatRes langChainChatRes = langChainChatService.deleteFile(DeleteFileDTO.create(modelCase.getCaseNo(), fileName));
                if (!langChainChatRes.getCode().equals(200)) {
                    log.error("删除文件:{} 失败!", fileName);
                }
            }
        }

    }
}
