package com.supervision.prompt.thread;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.police.domain.NotePrompt;
import com.supervision.police.domain.TripleInfo;
import com.supervision.prompt.vo.ExecResultDTO;
import com.supervision.thread.TripleExtractThread;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.util.StopWatch;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

@Slf4j
public class PromptTestThread implements Callable<ExecResultDTO> {

    private final OllamaChatClient chatClient;

    private final Map<String, String> paramMap;

    private final String prompt;

    public PromptTestThread(OllamaChatClient chatClient, String prompt, Map<String, String> paramMap) {
        this.chatClient = chatClient;
        this.paramMap = paramMap;
        this.prompt = prompt;
    }

    @Override
    public ExecResultDTO call() {
        try {
            StopWatch stopWatch = new StopWatch();
            // 分析三元组
            stopWatch.start();
            String format = StrUtil.format(prompt, paramMap);
            ChatResponse call = chatClient.call(new Prompt(new UserMessage(format)));
            stopWatch.stop();
            String content = call.getResult().getOutput().getContent();
            log.info("耗时:{},分析的结果是:{}", stopWatch.getTotalTimeSeconds(), content);
            ExecResultDTO resultDTO = new ExecResultDTO();
            resultDTO.setResult(content);
            resultDTO.setSubmitPrompt(format);
            resultDTO.setQuestion(paramMap.get("question"));
            resultDTO.setAnswer(paramMap.get("answer"));

            return resultDTO;
        } catch (Exception e) {
            log.error("提取三元组出现错误", e);
        }
        return null;
    }

}
