package com.supervision.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.springframework.web.multipart.MultipartFile;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;

@Slf4j
public class WordReadUtil {

    public static String readWord(String filePath) {

        StringBuilder stringBuilder = new StringBuilder();
        // 使用 FileInputStream 打开文件
        try (FileInputStream fis = new FileInputStream(filePath)) {
            // 创建 XWPFDocument 对象
            XWPFDocument document = new XWPFDocument(fis);

            // 获取所有段落
            List<XWPFParagraph> paragraphs = document.getParagraphs();

            // 遍历所有段落并打印文本
            for (XWPFParagraph paragraph : paragraphs) {
                stringBuilder.append(paragraph.getText());
            }
        } catch (Exception e) {
            log.error("获取笔录内容失败,{}", filePath, e);
        }
        return stringBuilder.toString();

    }

    public static String readWord(MultipartFile file) {
        StringBuilder stringBuilder = new StringBuilder();
        // 使用 InputStream 打开文件
        try (InputStream in = file.getInputStream()) {
            // 创建 XWPFDocument 对象
            XWPFDocument document = new XWPFDocument(in);

            // 获取所有段落
            List<XWPFParagraph> paragraphs = document.getParagraphs();

            // 遍历所有段落并打印文本
            for (XWPFParagraph paragraph : paragraphs) {
                stringBuilder.append(paragraph.getText());
            }
        } catch (Exception e) {
            log.error("获取笔录内容失败,{}", file.getOriginalFilename(), e);
        }
        return stringBuilder.toString();
    }

    public static String readWord(InputStream inputStream) {
        StringBuilder stringBuilder = new StringBuilder();
        try (inputStream) {
            // 创建 XWPFDocument 对象
            XWPFDocument document = new XWPFDocument(inputStream);
            // 获取所有段落
            List<XWPFParagraph> paragraphs = document.getParagraphs();
            // 遍历所有段落并打印文本
            for (XWPFParagraph paragraph : paragraphs) {
                stringBuilder.append(paragraph.getText()).append("\n");
            }
        } catch (Exception e) {
            log.error("获取笔录内容失败",e);
        }
        return stringBuilder.toString();
    }

}
